using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using KairoEngine.Core;
using KairoEngine.Core.ModuleSystem;
using Sirenix.OdinInspector;

namespace KairoEngine.SoundtrackSystem
{
    [Serializable, HideReferenceObjectPicker]
    public class SoundtrackModule : GameModuleBase
    {
        public override string name => "Soundtrack Module";

        [FoldoutGroup("@name"), NonSerialized, ShowInInspector]
        public Playlist playlist;

        [FoldoutGroup("@name")] public int currentTrackIndex = 0;
        [FoldoutGroup("@name")] public bool playOnStart = true;
        [FoldoutGroup("@name")] public bool showDebug = false;

        public SoundtrackModule(GameConfig config) : base(config)
        {
            this.gameConfig = config;
            this.className = this.GetType().AssemblyQualifiedName;
            this.typeName = "SoundtrackModule";
        }

        public override void Load(Transform parent)
        {
            GameObject obj = new GameObject();
            obj.transform.parent = parent;
            obj.transform.position = new Vector3();
            obj.transform.rotation = Quaternion.identity;
            obj.name = "SoundtrackManager";
            SoundtrackManager manager = obj.AddComponent<SoundtrackManager>();
            manager.playlist = playlist;
            manager.audioSource1 = obj.AddComponent<AudioSource>();
            manager.audioSource2 = obj.AddComponent<AudioSource>();
            manager.audioSource1.playOnAwake = false;
            manager.audioSource2.playOnAwake = false;
            manager.playOnStart = this.playOnStart;
            manager.showDebug = this.showDebug;
            manager.currentTrackIndex = this.currentTrackIndex;
        }

        public override void Start()
        {
            
            
        }

        public override void Reset()
        {
            this.playlist = null;
            this.currentTrackIndex = 0;
            this.playOnStart = true;
            this.showDebug = false;
        }

        public override void Destroy()
        {
            
        }

        public static SoundtrackModule JSONToSoundtrackModule(string data)
        {
            try
            {
                return JsonUtility.FromJson<SoundtrackModule>(data);
            }
            catch (System.Exception e)
            {
                Debug.LogError($"Could not deserialize SoundtrackModule: \n{e}");
                return new SoundtrackModule(null);
            }
        }

        public override void OnBeforeSerialize(ObjectSerializer serializer) 
        { 
            if(playlist != null) serializer.AddScriptableObject("SoundtrackModule_Playlist", playlist);
        }

        public override void OnBeforeDeserialize(ObjectSerializer serializer) 
        { 
            playlist = (Playlist)serializer.GetScriptableObject("SoundtrackModule_Playlist");
        }
    }

}